/*
    Copyright (c) 2015 digitalSTROM AG, Zurich, Switzerland

    Author: Sergey 'Jin' Bostandzhyan <jin@dev.digitalstrom.org>

    This file is part of vdSM, imported from libdSvDC.

    vdSM is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, version 2 of the License.


    vdSM is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with digitalSTROM Server. If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef __VDSM_AVAHI_DISCOVERY_H__
#define __VDSM_AVAHI_DISCOVERY_H__

#include <stdint.h>

#include <avahi-client/client.h>
#include <avahi-client/lookup.h>
#include <avahi-client/publish.h>

#include <avahi-common/simple-watch.h>
#include <avahi-common/malloc.h>
#include <avahi-common/error.h>

#include "types.h"
#include "role.h"

typedef struct vdsm_avahi_handle
{
    AvahiSimplePoll *simple_poll;
    AvahiClient *client;
    AvahiServiceBrowser *browser;
    AvahiServiceBrowser *coordination_browser;
    AvahiEntryGroup *group;
    bool initialized;
    VDSMRole vdsm_role;
    uint16_t vdsm_port;
    char *vdsm_dsuid;
    char *name;
} vdsm_avahi_handle_t;

void vdsm_avahi_resolve_callback(
        AvahiServiceResolver *resolver,
        AVAHI_GCC_UNUSED AvahiIfIndex interface,
        AVAHI_GCC_UNUSED AvahiProtocol protocol,
        AvahiResolverEvent event,
        const char *name,
        const char *type,
        const char *domain,
        const char *host_name,
        const AvahiAddress *address,
        uint16_t port,
        AvahiStringList *txt,
        AvahiLookupResultFlags flags,
        AVAHI_GCC_UNUSED void* userdata);

void vdsm_avahi_browse_callback(
        AvahiServiceBrowser *browser,
        AvahiIfIndex interface,
        AvahiProtocol protocol,
        AvahiBrowserEvent event,
        const char *name,
        const char *type,
        const char *domain,
        AVAHI_GCC_UNUSED AvahiLookupResultFlags flags,
        void* userdata);

void vdsm_avahi_client_callback(
        AvahiClient *client,
        AvahiClientState state,
        AVAHI_GCC_UNUSED void * userdata);

void vdsm_collection_initialized_callback(bool collection_enabled);
void vdsm_whitelist_changed_callback();

#endif//__VDSM_AVAHI_DISCOVERY_H__
