/*
    Copyright (c) 2015 digitalSTROM AG, Zurich, Switzerland

    Authors: Christian Hitz, digitalSTROM AG <christian.hitz@digitalstrom.com>
             Michael Tross, digitalSTROM AG <michael.tross@digitalstrom.com>
             Reto Flütsch, digitalSTROM AG <reto.fluetsch@digitalstrom.com>

    This file is part of vdSM.

    vdSM is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, version 2 of the License.
    

    vdSM is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with vdSM. If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef SELECTA_H_
#define SELECTA_H_

#include "qp_port.h"
#include "scene.h"
#include "bsp.h"
#include "output_channel.h"

typedef struct SharedOpTimeEvtTag {
    QTimeEvt super;
    QSignal sig;
    uint8_t buttonId;
    uint8_t localZoneId;
    uint8_t groupId;
    bool apartmentWide;
    uint16_t deviceId;
    uint8_t outputChannel;
} SharedOpTimeEvt;

typedef struct StopTimerEvtTag {
    QTimeEvt super;
    uint8_t localZoneId;
    uint8_t groupId;
    uint8_t buttonId;
    uint8_t padding;
} StopTimerEvt;

#define Op_QTimeEvt_postIn(me_, act_, nTicks_, sig_, buttonId_, localZoneId_, groupId_, deviceId_, apartmentWide_, outputChannel_) do { \
    (me_)->sig = sig_; \
    (me_)->buttonId = Datamodel_ButtonId_ButtonGroupId(buttonId_); \
    (me_)->localZoneId = localZoneId_; \
    (me_)->groupId = groupId_; \
    (me_)->deviceId = deviceId_; \
    (me_)->apartmentWide = apartmentWide_; \
    (me_)->outputChannel = outputChannel_; \
    QTimeEvt_postIn(&((me_)->super), (QActive *)act_, nTicks_); \
} while (0)

#define Op_QTimeEvt_disarm(me_) do { \
    if (QTimeEvt_disarm(&((me_)->super))) { \
        (me_)->sig = 0; \
    } \
} while (0)

#define Op_QTimeEvt_rearm(me_, nTicks_) do { \
    QTimeEvt_rearm(&((me_)->super), nTicks_); \
} while (0)

enum {
    SELECT_A_PRIVATE_ID_UPSTREAM,
    SELECT_A_PRIVATE_ID_PROGRAMMING_MODE,
};

typedef enum {
    APARTMENT_STATE_PRESENT = 0,
    APARTMENT_STATE_ABSENT = 1,
    APARTMENT_STATE_UNKNOWN = 2,
} ApartmentState_t;


/**
 * numbers of short downstream messages sent
 */
#define SELECTA_NO_DOWNSTREAM_MESSAGE           (0)
#define SELECTA_SHORT_DOWNSTREAM_MESSAGE        (1)
#define SELECTA_LONG_DOWNSTREAM_MESSAGE         (2*SELECTA_SHORT_DOWNSTREAM_MESSAGE)

uint8_t SelectA_CallSceneZoneGroup(uint8_t localZoneId, uint8_t groupId, uint8_t sourceZoneId, uint8_t sourceGroupId, Scene scene, uint16_t originDevice);
void SelectA_CallSceneZoneGroupApartment(uint8_t localZoneId, uint8_t groupId, uint8_t sourceZoneId, uint8_t sourceGroupId, Scene scene, bool apartmentWide, uint16_t originDevice);
void SelectA_ForceCallSceneZoneGroup(uint8_t localZoneId, uint8_t groupId, uint8_t sourceZoneId, uint8_t sourceGroupId, Scene scene, uint16_t originDevice);
void SelectA_ForceCallSceneZoneGroupApartment(uint8_t localZoneId, uint8_t groupId, uint8_t sourceZoneId, uint8_t sourceGroupId, Scene scene, bool apartmentWide, uint16_t originDevice);
void SelectA_UndoSceneNumberBroadcast(uint8_t sourceZoneId, uint8_t sourceGroupId, Scene scene, uint16_t originDevice);
void SelectA_SetSceneLocalPrioZoneGroup(uint8_t localZoneId, uint8_t groupId, Scene scene, uint16_t originDevice);
uint8_t SelectA_CallSceneMinZoneGroup(uint8_t localZoneId, uint8_t groupId, Scene scene, uint16_t originDevice);
void SelectA_SaveSceneZoneGroup(uint8_t localZoneId, uint8_t groupId, Scene scene, uint16_t originDevice);
void SelectA_SceneDeviceCallScene(uint8_t localZoneId, uint8_t groupId, Scene scene, bool force, uint16_t originDevice);
void SelectA_CallOPCDimZoneGroup(uint8_t localZoneId, uint8_t groupId, OPCOperation operation, uint8_t channelId, uint16_t originDevice);

uint8_t SelectA_SendSceneZoneGroup(uint8_t localZoneId, uint8_t groupId, Scene scene, bool force);
void SelectA_SendSceneDevice(uint16_t deviceId, Scene scene);
void SelectA_SendUndoSceneNumberZoneGroup(uint8_t localZoneId, uint8_t groupId, Scene scene);
void SelectA_SendBlinkZoneGroup(uint8_t localZoneId, uint8_t groupId);
void SelectA_SendProgOnZoneGroup(uint8_t localZoneId, uint8_t groupId);
void SelectA_SendProgOnDevice(uint16_t deviceId);
void SelectA_SendProgOffZoneGroup(uint8_t localZoneId, uint8_t groupId);
void SelectA_SendProgOffDevice(uint16_t deviceId);
void SelectA_SendSaveSceneZoneGroup(uint8_t localZoneId, uint8_t groupId, Scene scene);
void SelectA_SendOPCDimZoneGroup(uint8_t localZoneId, uint8_t groupId, OPCOperation operation, uint8_t channelId);
void SelectA_SendOPCDimDevice(uint16_t deviceId, OPCOperation operation, uint8_t channelId);
void SelectA_SendGeneralAck(void);

/**
 * @return the allocated timer
 */
StopTimerEvt* SelectA_AllocateStopTimer(uint8_t localZoneId, uint8_t groupId, uint8_t buttonId);
void SelectA_ReleaseStopTimer(StopTimerEvt *timer);
StopTimerEvt const * SelectA_GetStopTimerByIndex(uint8_t index);
bool SelectA_ButtonGroupTimerRunning(uint8_t localZoneId, uint8_t groupId);
bool SelectA_RoomTimerRunning(uint8_t localZoneId, uint8_t groupId);
void SelectA_StopRoomTimer(uint8_t localZoneId, uint8_t groupId);
void SelectA_PostGroupSignal(uint8_t groupId, uint8_t localZoneId, QSignal signal, bool force);

void SelectA_InitiateDevicePairing(uint16_t deviceId, uint8_t localZoneId);
void SelectA_RegisterLocalPrio(uint8_t localZoneId, uint8_t groupId, uint8_t buttonId, uint16_t deviceId);
void SelectA_UnregisterLocalPrio(uint8_t localZoneId, uint8_t groupId, uint8_t buttonId);
void SelectA_HandleRegisteredLocalPrio(uint8_t localZoneId, uint8_t groupId, uint16_t deviceId);
Scene SelectA_SecurityAlarmScene();
Scene SelectA_LightAlarmScene();
ApartmentState_t SelectA_GetApartmentState();

void SelectA_ctor(void);

extern QActive * const AO_SelectA; /* "opaque" pointer to Select A AO */

/**
 * Return margin of deferred event queue
 */
uint8_t SelectA_GetDeferredQueueMargin(void);
uint32_t SelectA_GetBinaryInputEventCount(void);


/*--------------------------------------------------------------------------*/
/* Sensors -----------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

enum SensorTypes {
    SENSOR_TYPE_ROOM_TEMPERATURE                = 9,
    SENSOR_TYPE_ROOM_TEMPERATURE_REQUESTED      = 50,
    SENSOR_TYPE_ROOM_TEMPERATURE_CONTROL_VALUE  = 51,
};

enum {
    MAX_CACHED_SENSORS      = 3,        /**< number of cached sensor types */
    SENSOR_INVALID_VALUE    = 0x7fff,   /**< means that real values is unknown */
    SENSOR_INVALID_INDEX    = 0xff,     /**< sensor type is not cached */
    SENSOR_MAX_AGE          = 0xffffffff,
};

typedef struct {
    uint32_t timestamp;
    uint16_t value;
    uint16_t padding;
} Sensor;

/**
 * Check sensor value for SENSOR_INVALID_VALUE
 */
Sensor SelectA_SensorsGetSensor(uint8_t localZoneId, uint8_t sensorType);
uint32_t SelectA_SensorsGetAge(uint32_t timestamp);

#endif /* SELECTA_H_ */
